package com.example.garrettmoody.facerecognitionapp;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Matrix;
import android.media.ExifInterface;
import android.media.FaceDetector;
import android.media.FaceDetector.Face;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;


public class FaceGuess extends Activity {
    public ArrayList<Face> faces = new ArrayList<>();
    public ArrayList<FaceNamePair> faceNamePairArrayList;
    public Uri imageUri;
    public Bitmap photo;
    public Button.OnClickListener noButtonListener = new Button.OnClickListener() {

        @Override
        public void onClick(View v) {
            Intent intent = new Intent(getApplicationContext(), matchNameWithFace.class);
            intent.putExtra("imageUri", imageUri);
            startActivity(intent);
        }
    };
    public FaceNamePair guessedAccount;
    public Button.OnClickListener yesButtonListener = new Button.OnClickListener() {

        @Override
        public void onClick(View v) {
            FaceNamePair addedAccount = new FaceNamePair(guessedAccount.getName(), guessedAccount.getEyeDistance(), imageUri.toString(), guessedAccount.getID());
            faceNamePairArrayList.add(addedAccount);
            saveFaceNamePairsToFile();

            Intent intent = new Intent(getApplicationContext(), MainActivity.class);
            startActivity(intent);
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_face_guess);

        Button yesButton = (Button) findViewById(R.id.yesButton);
        yesButton.setOnClickListener(yesButtonListener);

        Button noButton = (Button) findViewById(R.id.noButton);
        noButton.setOnClickListener(noButtonListener);

        TextView guessName = (TextView) findViewById(R.id.faceGuessText);

        imageUri = getIntent().getExtras().getParcelable("imageUri");
        photo = grabImage(imageUri);

        int orientation = getOrientationFromExif(imageUri.getPath());
        Matrix matrix = new Matrix();
        matrix.postRotate(orientation);
        Bitmap rotatedBitmap = Bitmap.createBitmap(photo, 0, 0, photo.getWidth(), photo.getHeight(), matrix, true);

        photo = rotatedBitmap;

        ImageView imageView = (ImageView) findViewById(R.id.guessImageView);
        imageView.setImageBitmap(photo);

        List<FaceNamePair> faceNamePairList = loadFaceNamePairsFromFile();
        if (faceNamePairList != null) {
            faceNamePairArrayList = new ArrayList<>(faceNamePairList.size());
            faceNamePairArrayList.addAll(faceNamePairList);
        } else {
            faceNamePairArrayList = new ArrayList<>(0);
        }

        guessedAccount = guessFace();
        guessName.setText("Is this " + guessedAccount.getName() + "?");

        ImageView imageThumbnail = (ImageView) findViewById(R.id.imageThumbnail);
        Uri guessedUri = Uri.parse(guessedAccount.getUriString());
        Bitmap guessedPhoto = grabImage(guessedUri);
        orientation = getOrientationFromExif(guessedUri.getPath());
        matrix = new Matrix();
        matrix.postRotate(orientation);
        Bitmap guessedThumbnail = Bitmap.createBitmap(guessedPhoto, 0, 0, guessedPhoto.getWidth(), guessedPhoto.getHeight(), matrix, true);
        guessedThumbnail = Bitmap.createScaledBitmap(guessedThumbnail, 40, 60, false);
        imageThumbnail.setImageBitmap(guessedThumbnail);
    }

    //Getting orientation from ExifInterface
    private static int getOrientationFromExif(String imagePath) {
        int orientation = -1;
        try {
            ExifInterface exif = new ExifInterface(imagePath);
            int exifOrientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL);
            Log.i("Orientation from Exif: ", exifOrientation + "");
            switch (exifOrientation) {
                case ExifInterface.ORIENTATION_ROTATE_270:
                    orientation = 270;
                    Log.i("Orientation from Exif", "270");
                    break;
                case ExifInterface.ORIENTATION_ROTATE_180:
                    orientation = 180;
                    Log.i("Orientation from Exif", "180");
                    break;
                case ExifInterface.ORIENTATION_ROTATE_90:
                    Log.i("Orientation from Exif", "90");
                    orientation = 90;
                    break;
                case ExifInterface.ORIENTATION_NORMAL:
                    orientation = 0;
                    Log.i("Orientation from Exif", "0 - Normal");
                    break;
                case ExifInterface.ORIENTATION_UNDEFINED:
                    orientation = -1;
                    Log.e("Orientation from Exif", "UNDEFINED");
            }
        } catch (IOException e) {
        }
        return orientation;
    }

    public Bitmap grabImage(Uri uriSavedImage) {
        Bitmap bitmap = null;
        try {
            bitmap = MediaStore.Images.Media.getBitmap(this.getContentResolver(), uriSavedImage);
        } catch (Exception e) {
            Toast.makeText(this, "Failed to load", Toast.LENGTH_SHORT).show();
        }

        return bitmap;
    }

    private FaceNamePair guessFace() {
        Face newFace = findFaceInPhoto();
        float newEyeDistance = newFace.eyesDistance();
        int smallestDistanceIndex = 0;
        float smallestDistance = Math.abs(newEyeDistance - faceNamePairArrayList.get(0).getEyeDistance());
        for (int i = 0; i < faceNamePairArrayList.size(); i++) {
            float testEyeDistance = Math.abs(newEyeDistance - faceNamePairArrayList.get(i).getEyeDistance());
            if (testEyeDistance < smallestDistance) {
                smallestDistance = testEyeDistance;
                smallestDistanceIndex = i;
            }
        }
        FaceNamePair guessedFace = new FaceNamePair(faceNamePairArrayList.get(smallestDistanceIndex).getName(), newFace.eyesDistance(),
                faceNamePairArrayList.get(smallestDistanceIndex).getUriString(), faceNamePairArrayList.get(smallestDistanceIndex).getID());

        return guessedFace;
    }

    public FaceDetector.Face findFaceInPhoto() {
        FaceDetector.Face face[] = new FaceDetector.Face[1];

        Bitmap newPhoto = photo.copy(Bitmap.Config.RGB_565, true);

        FaceDetector faceDetector;
        faceDetector = new FaceDetector(newPhoto.getWidth(), newPhoto.getHeight(), 1);
        int facesFound = faceDetector.findFaces(newPhoto, face);

        if (facesFound < 1) {
            Toast.makeText(getApplicationContext(), "No faces found.", Toast.LENGTH_LONG);
        } else if (facesFound > 1) {
            Toast.makeText(getApplicationContext(), "Too many faces found.", Toast.LENGTH_LONG);
        }
        return face[0];
    }

    public void saveFaceNamePairsToFile() {
        Context context = getApplicationContext();
        File faceNamePairsFile = new File(context.getFilesDir(), "FaceNamePairs.ser");

        String filename = faceNamePairsFile.getPath();
        FileOutputStream fos = null;
        ObjectOutputStream out = null;
        try {
            fos = new FileOutputStream(filename);
            out = new ObjectOutputStream(fos);
            for (int i = 0; i < faceNamePairArrayList.size(); i++) {
                out.writeFloat(faceNamePairArrayList.get(i).getEyeDistance());
                out.writeInt(faceNamePairArrayList.get(i).getID());
                out.writeObject(faceNamePairArrayList.get(i).getName());
                out.writeObject(faceNamePairArrayList.get(i).getUriString());
            }

            out.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public List<FaceNamePair> loadFaceNamePairsFromFile() {
        Context context = getApplicationContext();
        File contactsFile = new File(context.getFilesDir(), "FaceNamePairs.ser");
        if (!contactsFile.exists() || contactsFile.length() == 0) {
            List<FaceNamePair> emptyList = null;
            return emptyList;
        }

        List<FaceNamePair> loadedFaceNamePairs = new ArrayList<>();

        FileInputStream fileInputStream;
        try {
            fileInputStream = new FileInputStream(contactsFile);
        } catch (FileNotFoundException e) {
            Toast.makeText(getApplicationContext(), "Nothing to load.", Toast.LENGTH_LONG);
            return new ArrayList<>();
        }

        try {
            ObjectInputStream ois = new ObjectInputStream(fileInputStream);
            boolean again = true;
            while (again) {
                float eyeDistance = ois.readFloat();
                int ID = ois.readInt();
                String name = (String) ois.readObject();
                String Uri = (String) ois.readObject();
                FaceNamePair newFace = new FaceNamePair(name, eyeDistance, Uri, ID);
                loadedFaceNamePairs.add(newFace);
                if (newFace == null)
                    again = false;
            }

            ois.close();
            fileInputStream.close();
        } catch (EOFException e)
        //ignore
        {
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace();
        }

        return loadedFaceNamePairs;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_face_analysis, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
