package com.example.garrettmoody.facerecognitionapp;

import android.app.ListActivity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Matrix;
import android.media.ExifInterface;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.Toast;

import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

public class PhotoGallery extends ListActivity
{
    List<FaceNamePair> list = new ArrayList<>();

    public void onCreate(Bundle saveInstanceState) {
        super.onCreate(saveInstanceState);
        setContentView(R.layout.activity_photo_gallery);
        list = loadFaceNamePairsFromFile();

        Button deleteButton = (Button)findViewById(R.id.btnDelete);
        deleteButton.setOnClickListener(deleteButtonOnClickListener);

        Bitmap[] bitmapValues = new Bitmap[list.size()];
        String[] values = new String[list.size()];
        Float[] eyeDistanceValues = new Float[list.size()];
        for (int i = 0; i < list.size(); i++) {
            //grab the name
            values[i] = list.get(i).getName();
            eyeDistanceValues[i] = list.get(i).getEyeDistance();

            //grab the photos and make sure they are properly oriented
            Uri imageUri = Uri.parse(list.get(i).getUriString());
            Bitmap photo = grabImage(imageUri);
            int orientation = getOrientationFromExif(imageUri.getPath());
            Matrix matrix = new Matrix();
            matrix.postRotate(orientation);
            Bitmap rotatedBitmapThumbnail = Bitmap.createBitmap(photo, 0, 0, photo.getWidth(), photo.getHeight(), matrix, true);
            int THUMBNAIL_HEIGHT = 450;
            int THUMBNAIL_WIDTH = 300;
            rotatedBitmapThumbnail = Bitmap.createScaledBitmap(rotatedBitmapThumbnail, THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT, false);
            bitmapValues[i] = rotatedBitmapThumbnail;
        }

        ArrayAdapter<FaceNamePair> adapter = new MySimpleArrayAdapter(this,
                list, bitmapValues);
        setListAdapter(adapter);
    }

    Button.OnClickListener deleteButtonOnClickListener = new Button.OnClickListener(){

        @Override
        public void onClick(View v) {
            boolean restart = false;
            for(int i = 0; i < list.size(); i++)
            {
                if(list.get(i).isSelected())
                {
                    list.remove(i);
                    i--;
                    restart = true;
                }
            }
            saveFaceNamePairsToFile();
            if(restart) {
                Intent intent = getIntent();
                startActivity(intent);
            }
        }
    };

    @Override
    protected void onListItemClick(ListView l, View v, int position, long id) {
        String item = (String) getListAdapter().getItem(position);
        Toast.makeText(this, item + " selected", Toast.LENGTH_LONG).show();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_photo_gallery, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    public List<FaceNamePair> loadFaceNamePairsFromFile() {
        Context context = getApplicationContext();
        File contactsFile = new File(context.getFilesDir(), "FaceNamePairs.ser");
        if (!contactsFile.exists() || contactsFile.length() == 0) {
            List<FaceNamePair> emptyList = null;
            return emptyList;
        }

        List<FaceNamePair> loadedFaceNamePairs = new ArrayList<>();

        FileInputStream fileInputStream;
        try {
            fileInputStream = new FileInputStream(contactsFile);
        } catch (FileNotFoundException e) {
            Toast.makeText(getApplicationContext(), "Nothing to load.", Toast.LENGTH_LONG);
            return new ArrayList<>();
        }

        try {
            ObjectInputStream ois = new ObjectInputStream(fileInputStream);
            boolean again = true;
            while (again) {
                float eyeDistance = ois.readFloat();
                int ID = ois.readInt();
                String name = (String) ois.readObject();
                String Uri = (String) ois.readObject();
                FaceNamePair newFace = new FaceNamePair(name, eyeDistance, Uri, ID);
                loadedFaceNamePairs.add(newFace);
                if (newFace == null)
                    again = false;
            }

            ois.close();
            fileInputStream.close();
        } catch (EOFException e)
        //ignore
        {
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace();
        }

        return loadedFaceNamePairs;
    }

    //Getting orientation from ExifInterface
    private static int getOrientationFromExif(String imagePath) {
        int orientation = -1;
        try {
            ExifInterface exif = new ExifInterface(imagePath);
            int exifOrientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL);
            Log.i("Orientation from Exif: ", exifOrientation + "");
            switch (exifOrientation) {
                case ExifInterface.ORIENTATION_ROTATE_270:
                    orientation = 270;
                    Log.i("Orientation from Exif", "270");
                    break;
                case ExifInterface.ORIENTATION_ROTATE_180:
                    orientation = 180;
                    Log.i("Orientation from Exif", "180");
                    break;
                case ExifInterface.ORIENTATION_ROTATE_90:
                    Log.i("Orientation from Exif", "90");
                    orientation = 90;
                    break;
                case ExifInterface.ORIENTATION_NORMAL:
                    orientation = 0;
                    Log.i("Orientation from Exif", "0 - Normal");
                    break;
                case ExifInterface.ORIENTATION_UNDEFINED:
                    orientation = -1;
                    Log.e("Orientation from Exif", "UNDEFINED");
            }
        } catch (IOException e) {
        }
        return orientation;
    }

    public Bitmap grabImage(Uri uriSavedImage) {
        Bitmap bitmap = null;
        try {
            bitmap = MediaStore.Images.Media.getBitmap(this.getContentResolver(), uriSavedImage);
        } catch (Exception e) {
            Toast.makeText(this, "Failed to load", Toast.LENGTH_SHORT).show();
        }

        return bitmap;
    }

    public void saveFaceNamePairsToFile() {
        Context context = getApplicationContext();
        File faceNamePairsFile = new File(context.getFilesDir(), "FaceNamePairs.ser");

        String filename = faceNamePairsFile.getPath();
        FileOutputStream fos = null;
        ObjectOutputStream out = null;
        try {
            fos = new FileOutputStream(filename);
            out = new ObjectOutputStream(fos);
            for (int i = 0; i < list.size(); i++) {
                out.writeFloat(list.get(i).getEyeDistance());
                out.writeInt(list.get(i).getID());
                out.writeObject(list.get(i).getName());
                out.writeObject(list.get(i).getUriString());
            }

            out.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}

